/*
INTEL CONFIDENTIAL

Copyright 2013-2015 Intel Corporation All Rights Reserved.
The source code contained or described herein and all documents related
to the source code ("Material") are owned by Intel Corporation or its
suppliers or licensors. Title to the Material remains with Intel Corporation
or its suppliers and licensors. The Material contains trade secrets and
proprietary and confidential information of Intel or its suppliers and
licensors. The Material is protected by worldwide copyright and trade secret
laws and treaty provisions. No part of the Material may be used, copied,
reproduced, modified, published, uploaded, posted, transmitted, distributed,
or disclosed in any way without Intel's prior express written permission.

No license under any patent, copyright, trade secret or other intellectual
property right is granted to or conferred upon you by disclosure or delivery
of the Materials, either expressly, by implication, inducement, estoppel or
otherwise. Any license under such intellectual property rights must be
express and approved by Intel in writing.
*/

'use strict';

(function (global, factory) {
  if (typeof define === "function" && define.amd) {
    define(['exports'], factory);
  } else if (typeof exports !== "undefined") {
    factory(exports);
  } else {
    var mod = {
      exports: {}
    };
    factory(mod.exports);
    global.fp = mod.exports;
  }
})(this, function (exports) {
  Object.defineProperty(exports, "__esModule", {
    value: true
  });
  exports.curry = curry;
  exports.flow = flow;
  exports.flowLens = flowLens;
  exports.pathLens = pathLens;
  exports.cond = cond;
  exports.shallowClone = shallowClone;
  exports.unwrap = unwrap;
  exports.once = once;

  var __ = exports.__ = {};

  function _type(val) {
    if (val === null) return 'Null';else if (val === undefined) return 'Undefined';else return Object.prototype.toString.call(val).slice(8, -1);
  }

  function curry(n, fn) {
    return function innerCurry() {
      var args = new Array(arguments.length);

      for (var i = 0, l = arguments.length; i < l; i++) {
        args[i] = arguments[i];
      }

      var gaps = n - args.length;
      gaps += args.filter(function findGaps(x) {
        return x === __;
      }).length;
      if (gaps > 0) return curry(gaps, function () {
        var innerArgs = new Array(arguments.length);

        for (var i = 0, l = arguments.length; i < l; i++) {
          innerArgs[i] = arguments[i];
        }

        var filledArgs = args.map(function (x) {
          if (x === __) return innerArgs.shift();
          return x;
        });
        return fn.apply(null, filledArgs.concat(innerArgs).slice(0, n));
      });else return fn.apply(null, args.slice(0, n));
    };
  }

  var map = exports.map = curry(2, function m(f, x) {
    if (x && typeof x.map === 'function') return x.map(curry(1, f));else return f(x);
  });
  var filter = exports.filter = curry(2, function filter(f, xs) {
    if (xs && typeof xs.filter === 'function') return xs.filter(curry(1, f));else return f(xs);
  });
  var reduce = exports.reduce = curry(3, function reducer(accum, f, xs) {
    if (typeof accum === 'function') accum = accum();

    if (Array.isArray(xs)) return xs.reduce(curry(2, f), accum);
    if (xs && typeof xs.reduce === 'function') return xs.reduce(accum, curry(2, f));else return f(accum, xs);
  });
  var deepEq = exports.deepEq = curry(2, function deepEqFactory(a, b) {
    return (function deepEq(a, b, visitedA, visitedB) {

      var aType = _type(a);
      var bType = _type(b);

      if (isColl(aType) && isColl(bType) && aType === bType) {
        // Collection check.

        // Cycle checks.
        throwForCycle(visitedA, a);
        throwForCycle(visitedB, b);

        var aKeys = Object.keys(a);
        var bKeys = Object.keys(b);

        if (aKeys.length !== bKeys.length) // Length check.
          return false;

        var keysInBoth = aKeys.every(function aInB(x) {
          return x in b;
        });

        if (!keysInBoth) return false;

        return aKeys.every(function recurse(x) {
          // Recursive check.
          return deepEq(a[x], b[x], visitedA, visitedB);
        });
      } else {
        return eq(a, b); // Strict check.
      }

      function isColl(t) {
        return t === 'Array' || t === 'Object';
      }

      function throwForCycle(visited, xs) {
        if (visited.indexOf(xs) !== -1) throw new Error('Cycle detected, cannot determine equality.');

        visited.push(xs);
      }
    })(a, b, [], []);
  });
  var find = exports.find = curry(2, function find(f, xs) {
    return filter(f, xs)[0];
  });
  var pluck = exports.pluck = curry(2, function pluck(key, xs) {
    return map(function plucker(xs) {
      return xs[key];
    }, xs);
  });

  var identity = exports.identity = function identity(x) {
    return x;
  };

  var always = exports.always = function always(x) {
    return function () {
      return x;
    };
  };

  var True = exports.True = always(true);
  var difference = exports.difference = curry(2, function difference(xs, ys) {
    return xs.reduce(function reduceDiffs(arr, x) {
      if (ys.indexOf(x) === -1) arr.push(x);

      return arr;
    }, []);
  });
  var lens = exports.lens = curry(2, function lens(get, set) {
    set = curry(2, set);
    innerLens.set = set;

    innerLens.map = curry(2, function map(fn, xs) {
      return flow(get, fn, set(__, xs))(xs);
    });

    function innerLens(x) {
      return get(x);
    }

    return innerLens;
  });

  var lensProp = exports.lensProp = function lensProp(prop) {
    return lens(function get(x) {
      return x[prop];
    }, function set(val, x) {
      x[prop] = val;

      return x;
    });
  };

  function flow() {
    var args = new Array(arguments.length);

    for (var i = 0, l = arguments.length; i < l; i++) {
      args[i] = arguments[i];
    }

    return function flowInner(xs) {
      return args.reduce(function reducer(xs, fn) {
        return fn(xs);
      }, xs);
    };
  }

  var flowN = exports.flowN = curry(2, function flowN(n, fns) {
    fns = lensProp('0').map(invoke, fns);
    var wrappedFlow = wrapArgs(flow.apply(null, fns));

    return curry(n, wrappedFlow);
  });

  function flowLens() {
    var args = new Array(arguments.length);

    for (var i = 0, l = arguments.length; i < l; i++) {
      args[i] = arguments[i];
    }

    return lens(flow.apply(null, args), function set(val, xs) {
      args.reduce(function reducer(xs, l, index) {
        if (index === args.length - 1) return l.set(val, xs);else return l(xs);
      }, xs);
      return xs;
    });
  }

  function pathLens(path) {
    var pathType = _type(path);

    if (pathType !== 'Array') throw new TypeError('pathLens must receive the path in the form of an array. Got: ' + pathType);
    var lenses = map(lensProp, path);
    return lens(safe(1, flow.apply(null, lenses), undefined), function set(val, xs) {
      lenses.reduce(function reducer(xs, l, index) {
        if (index === lenses.length - 1) return l.set(val, xs);
        var x = l(xs);
        if (x != null) return x;
        x = {};
        l.set(x, xs);
        return x;
      }, xs);
      return xs;
    });
  }

  function cond() {
    var args = new Array(arguments.length);

    for (var i = 0, l = arguments.length; i < l; i++) {
      args[i] = arguments[i];
    }

    return function condInner(xs) {
      var result;
      args.some(function findTruthy(pair) {
        if (pair[0](xs)) return result = pair[1](xs);
      });
      return result;
    };
  }

  function shallowClone(xs) {
    var type = _type(xs);

    if (type === 'Array') return xs.slice();else if (type === 'Object') return Object.keys(xs).reduce(function reducer(obj, key) {
      obj[key] = xs[key];
      return obj;
    }, {});
    throw new Error('shallow clone works with Array or Object. Got: ' + type);
  }

  var not = exports.not = function not(x) {
    return !x;
  };

  var eq = exports.eq = curry(2, function (a, b) {
    return a === b;
  });
  var eqLens = exports.eqLens = curry(3, function eqLens(l, a, b) {
    return eq(l(a), l(b));
  });
  var eqFn = exports.eqFn = curry(4, function eqFn(fnA, fnB, a, b) {
    return eq(fnA(a), fnB(b));
  });
  var invoke = exports.invoke = curry(2, function invoke(fn, args) {
    if (!Array.isArray(args)) throw new Error('Error in fp.invoke - Cannot call invoke with non-array');

    return fn.apply(null, args);
  });
  var safe = exports.safe = curry(3, function safe(arity, fn, def) {
    return curry(arity, function safeCheck() {
      for (var i = 0, l = arguments.length; i < l; i++) {
        if (arguments[i] == null) return def;
      }

      try {
        return fn.apply(null, arguments);
      } catch (e) {
        return def;
      }
    });
  });

  var noop = exports.noop = function noop() {};

  var and = exports.and = curry(2, function and(predicates, val) {
    return predicates.reduce(function reducer(curr, predicate) {
      return curr && predicate(val);
    }, true);
  });
  var or = exports.or = curry(2, function or(predicates, val) {
    return predicates.reduce(function reducer(curr, predicate) {
      return curr || predicate(val);
    }, false);
  });
  var bindMethod = exports.bindMethod = curry(2, function bindMethod(meth, obj) {
    return obj[meth].bind(obj);
  });
  var invokeMethod = exports.invokeMethod = curry(3, function invokeMethod(meth, args, obj) {
    return flow(bindMethod(meth), invoke(__, args))(obj);
  });
  var zipObject = exports.zipObject = curry(2, function zipObject(keys, vals) {
    var keysType = _type(keys);
    var valsType = _type(vals);
    if (keysType !== 'Array') throw new TypeError('zipObject keys must be an Array. Got: ' + keysType);
    if (valsType !== 'Array') throw new TypeError('zipObject values must be an Array. Got: ' + valsType);

    return keys.reduce(function reduceToObject(obj, val, index) {
      obj[val] = vals[index];
      return obj;
    }, {});
  });
  var some = exports.some = curry(2, function some(fn, xs) {
    if (xs && typeof xs.some === 'function') return xs.some(curry(1, fn));else return !!fn(xs);
  });
  var every = exports.every = curry(2, function every(fn, xs) {
    if (xs && typeof xs.every === 'function') return xs.every(curry(1, fn));else return !!fn(xs);
  });

  function unwrap(xs) {
    return xs.reduce(function reducer(arr, x) {
      return arr.concat(x);
    }, []);
  }

  var head = exports.head = function head(xs) {
    return xs[0];
  };

  var tail = exports.tail = flow(invokeMethod('slice', [-1]), head);

  var arrayWrap = exports.arrayWrap = function arrayWrap(x) {
    return [x];
  };

  function once(fn) {
    var called = false;
    return function innerOnce() {
      if (called) return;
      called = true;
      fn.apply(null, arguments);
    };
  }

  var either = exports.either = curry(2, function either(fn, x) {
    if (x instanceof Error) return x;

    return fn(x);
  });
  var unsafe = exports.unsafe = curry(2, function unsafe(fn, x) {
    if (!(x instanceof Error)) return x;

    return fn(x);
  });
  var tap = exports.tap = curry(2, function tap(fn, xs) {
    fn(xs);
    return xs;
  });
  var mapFn = exports.mapFn = curry(2, function mapFn(fns, args) {
    return map(invoke(__, args), fns);
  });
  var chainL = exports.chainL = curry(2, function chainL(fn, args) {
    return args.reduce(curry(2, fn));
  });

  var wrapArgs = exports.wrapArgs = function wrapArgs(fn) {
    return function innerWrapArgs() {
      return fn.call(null, [].slice.call(arguments));
    };
  };

  var xProd = exports.xProd = curry(2, function xprod(a, b) {
    var result = [];

    a.forEach(function (a) {
      b.forEach(function (b) {
        result.push([a, b]);
      });
    });

    return result;
  });
  var flip = exports.flip = curry(2, function flip(arity, fn) {
    return curry(arity, function flipper() {
      var args = new Array(arguments.length);

      for (var i = 0, l = arguments.length; i < l; i++) {
        args[i] = arguments[i];
      }
      args.reverse();
      return fn.apply(null, args);
    });
  });
  var anyPass = exports.anyPass = curry(2, function anyPass(fns, x) {
    return some(invoke(__, [x]), fns);
  });
  var zipBy = exports.zipBy = curry(3, function (fn, left, right) {
    var min = Math.min(left.length, right.length);
    left.length = min;
    right.length = min;

    return left.reduce(function (prev, cur, idx) {
      return prev.concat(fn(cur, right[idx]));
    }, []);
  });
});